/* Distributed under the OSI-approved BSD 3-Clause License.  See accompanying
   file Copyright.txt or https://cmake.org/licensing for details.  */
#ifndef cmVersionComparison_h
#define cmVersionComparison_h

#include "cmConfigure.h" // IWYU pragma: keep

#include <string>

namespace cmVersionComparison {

enum Operator
{
  AlwaysMatch = 0,
  Equal = 1,
  Less = 2,
  Greater = 4,
  LessEqual = Less | Equal,
  GreaterEqual = Greater | Equal,
  NotEqual = Less | Greater
};

/**
 * Compare dot separated segments of dicimal numbers.
 */
int Compare(std::string const& lhs, std::string const& rhs);
bool Match(std::string const& lhs, Operator op, std::string const& rhs);

struct CompareGreater
{
  bool operator()(std::string const& lhs, std::string const& rhs) const
  {
    return Compare(lhs, rhs) > 0;
  }
};

/**
 * Debian type version compare.
 */
int CompareDEB(std::string const& lhs, std::string const& rhs);
bool MatchDEB(std::string const& lhs, Operator op, std::string const& rhs);

/**
 * RPM type version compare.
 */
int CompareRPM(std::string const& lhs, std::string const& rhs);
bool MatchRPM(std::string const& lhs, Operator op, std::string const& rhs);

/**
 * Compare two ASCII strings using natural versioning order.
 * Non-numerical characters are compared directly.
 * Numerical characters are first globbed such that, e.g.
 * `test000 < test01 < test0 < test1 < test10`.
 * Return a value less than, equal to, or greater than zero if lhs
 * precedes, equals, or succeeds rhs in the defined ordering.
 */
int strverscmp(std::string const& lhs, std::string const& rhs);

} // namespace cmVersionComparison

#endif
