/* Distributed under the OSI-approved BSD 3-Clause License.  See accompanying
   file Copyright.txt or https://cmake.org/licensing for details.  */

#include <cmConfigure.h> // IWYU pragma: keep

#include <iostream>
#include <stddef.h>
#include <string>
#include <vector>

#include "cmVersionComparison.h"

#define cmPassed(m) std::cout << "Passed: " << (m) << "\n"
#define cmFailed(m)                                                           \
  std::cout << "FAILED: " << (m) << "\n";                                     \
  failed = 1

#define cmAssert(exp, m)                                                      \
  if ((exp)) {                                                                \
    cmPassed(m);                                                              \
  } else {                                                                    \
    cmFailed(m);                                                              \
  }

int testVersionComparison(int /*unused*/, char* /*unused*/ [])
{
  using cmVersionComparison::strverscmp;

  int failed = 0;
  // ----------------------------------------------------------------------
  // Test strverscmp
  cmAssert(strverscmp("", "") == 0, "strverscmp empty string");
  cmAssert(strverscmp("abc", "") > 0, "strverscmp string vs empty string");
  cmAssert(strverscmp("abc", "abc") == 0, "strverscmp same string");
  cmAssert(strverscmp("abd", "abc") > 0, "strverscmp character string");
  cmAssert(strverscmp("abc", "abd") < 0, "strverscmp symmetric");
  cmAssert(strverscmp("12345", "12344") > 0, "strverscmp natural numbers");
  cmAssert(strverscmp("100", "99") > 0,
           "strverscmp natural numbers different digits");
  cmAssert(strverscmp("12345", "00345") > 0,
           "strverscmp natural against decimal (same length)");
  cmAssert(strverscmp("99999999999999", "99999999999991") > 0,
           "strverscmp natural overflow");
  cmAssert(strverscmp("00000000000009", "00000000000001") > 0,
           "strverscmp deciaml precision");
  cmAssert(strverscmp("a.b.c.0", "a.b.c.000") > 0,
           "strverscmp multiple zeros");
  cmAssert(strverscmp("lib_1.2_10", "lib_1.2_2") > 0,
           "strverscmp last number ");
  cmAssert(strverscmp("12lib", "2lib") > 0, "strverscmp first number ");
  cmAssert(strverscmp("02lib", "002lib") > 0,
           "strverscmp first number decimal ");
  cmAssert(strverscmp("10", "9a") > 0, "strverscmp letter filler ");
  cmAssert(strverscmp("000", "0001") > 0,
           "strverscmp zero and leading zeros  ");

  // test sorting using standard strvercmp input
  std::vector<std::string> testString;
  testString.push_back("000");
  testString.push_back("00");
  testString.push_back("01");
  testString.push_back("010");
  testString.push_back("09");
  testString.push_back("0");
  testString.push_back("1");
  testString.push_back("9");
  testString.push_back("10");

  // test global ordering of input strings
  for (size_t i = 0; i < testString.size() - 1; i++) {
    for (size_t j = i + 1; j < testString.size(); j++) {
      if (strverscmp(testString[i], testString[j]) >= 0) {
        cmFailed("strverscmp error in comparing strings " + testString[i] +
                 " " + testString[j]);
      }
    }
  }

  if (!failed) {
    cmPassed("strverscmp working");
  }
  return failed;
}
